from pathlib import Path
from typing import Optional

from pydantic import ConfigDict, BaseModel


class Git(BaseModel):
    uri: str
    branch: str = "main"
    paths: Optional[list[Path]] = None
    private_key: Optional[str] = None
    ignore_paths: Optional[list[Path]] = None
    model_config = ConfigDict(extra="allow")



from typing import Optional, Union

from ol_concourse.lib.models.pipeline import Identifier, RegistryImage, Resource
from ol_concourse.lib.models.resource import Git


def git_repo(  # noqa: PLR0913
    name: Identifier,
    uri: str,
    branch: str = "main",
    check_every: str = "60s",
    paths: Optional[list[str]] = None,
    depth: Optional[int] = None,
    **kwargs,
) -> Resource:
    return Resource(
        name=name,
        type="git",
        icon="git",
        check_every=check_every,
        source=Git(uri=uri, branch=branch, paths=paths).model_dump(exclude_none=True),
        **kwargs,
    )


def ssh_git_repo(
    name: Identifier,
    uri: str,
    private_key: str,
    branch: str = "main",
    paths: Optional[list[str]] = None,
) -> Resource:
    return Resource(
        name=name,
        type="git",
        icon="git",
        source=Git(
            uri=uri, branch=branch, paths=paths, private_key=private_key
        ).model_dump(exclude_none=True),
    )


def github_release(name: Identifier, owner: str, repository: str) -> Resource:
    """Generate a github-release resource for the given owner/repository.

    :param name: The name of the resource. This will get used across subsequent
        pipeline steps that reference this resource.
    :type name: Identifier
    :param owner: The owner of the repository (e.g. the GitHub user or organization)
    :type owner: str
    :param repository: The name of the repository as it appears in GitHub
    :type repository: str
    :returns: A configured Concourse resource object that can be used in a pipeline.
    :rtype: Resource
    """
    return Resource(
        name=name,
        type="github-release",
        icon="github",
        check_every="24h",
        source={"repository": repository, "owner": owner, "release": True},
    )


def hashicorp_release(name: Identifier, project: str) -> Resource:
    """Generate a hashicorp-release resource for the given application.  # noqa: DAR201

    :param name: The name of the resourc. This will get used across subsequent
        pipeline steps taht reference this resource.
    :type name: Identifier
    :param project: The name of the hashicorp project to check for a release of.
    :type project: str
    """
    return Resource(
        name=name,
        type="hashicorp-release",
        icon="lock-check",
        check_every="24h",
        source={"project": project},
    )


def amazon_ami(
    name: Identifier,
    filters: dict[str, Union[str, bool]],
    region: str = "us-east-1",
) -> Resource:
    return Resource(
        name=name,
        type="amazon-ami",
        icon="server",
        check_every="30m",
        source={
            "region": region,
            "filters": filters,
        },
    )


def pulumi_provisioner(
    name: Identifier, project_name: str, project_path: str
) -> Resource:
    return Resource(
        name=name,
        type="pulumi-provisioner",
        icon="cloud-braces",
        source={
            "env_pulumi": {"AWS_SHARED_CREDENTIALS_FILE": "aws_creds/credentials"},
            "action": "update",
            "project_name": project_name,
            "source_dir": project_path,
        },
    )


def schedule(name: Identifier, interval: str) -> Resource:
    return Resource(
        name=name,
        type="time",
        icon="clock",
        source={"interval": interval},
    )


def registry_image(  # noqa: PLR0913
    name: Identifier,
    image_repository: str,
    image_tag: Optional[str] = None,
    variant: Optional[str] = None,
    username=None,
    password=None,
) -> Resource:
    image_source = RegistryImage(
        repository=image_repository, tag=image_tag or "latest"
    ).model_dump()
    if username and password:
        image_source["username"] = username
        image_source["password"] = password
    if variant:
        image_source["variant"] = variant
    return Resource(
        name=name,
        type="registry-image",
        source=image_source,
    )


# https://github.com/arbourd/concourse-slack-alert-resource
# We use only a very basic implementation of this notification framework
def slack_notification(name: Identifier, url: str) -> Resource:
    return Resource(
        name=name, type="slack-notification", source={"url": url, "disabled": False}
    )



from typing import Optional

from ol_concourse.lib.jobs.infrastructure import Output
from ol_concourse.lib.models.pipeline import (
    Command,
    Identifier,
    Input,
    TaskConfig,
    TaskStep,
    Cache,
)


def container_build_task(
    inputs: list[Input],
    build_parameters: Optional[dict[str, str]],
    build_args: Optional[list[str]] = None,
) -> TaskStep:
    return TaskStep(
        task=Identifier("build-container-image"),
        privileged=True,
        config=TaskConfig(
            platform="linux",
            image_resource={
                "type": "registry-image",
                "source": {"repository": "concourse/oci-build-task"},
            },
            params=build_parameters,
            caches=[Cache(path="cache")],
            run=Command(
                path="build",
                args=build_args or [],
            ),
            inputs=inputs,
            # This output needs to be named exactly "image" or it won't actually export
            # the built image.
            outputs=[Output(name=Identifier("image"))],
        ),
    )



from typing import Optional

from ol_concourse.lib.models.pipeline import PutStep, Resource


def notification(
    resource: Resource,
    title: str,
    body: str,
    alert_type: Optional[str] = "default",
) -> PutStep:
    """Generate a PutStep for sending notifications (to just slack, for now).

    :param resource: The slack_notification_resource object for the pipeline.
        See src/ol_concourse/lib/resources.py
    :param title: The text to send to slack as the 'title' of the notification. The bold
        main line.
    :param body: The text to send to slack as the 'body' of the notification. Additional
        details that are hidden when the alert is collapsed.
    :param alert_type: The type of alert to send. Determined the color of the alert
        in slack.
        See: https://github.com/arbourd/concourse-slack-alert-resource#alert-types.
        At this time we don't support 'started' notifications.

    :returns: A `PutStep` object that can be executed as an 'on_success',
        'on_failure', etc
    """
    params = {"alert_type": alert_type, "message": title, "text": body}
    return PutStep(
        put=resource.name,
        params=params,
    )



from ol_concourse.lib.constants import REGISTRY_IMAGE
from ol_concourse.lib.models.pipeline import Identifier, RegistryImage, ResourceType


def hashicorp_resource() -> ResourceType:
    return ResourceType(
        name=Identifier("hashicorp-release"),
        type=REGISTRY_IMAGE,
        source=RegistryImage(repository="mitodl/hashicorp-release-resource"),
    )


def rclone() -> ResourceType:
    return ResourceType(
        name=Identifier("rclone"),
        type=REGISTRY_IMAGE,
        source=RegistryImage(repository="mitodl/concourse-rclone-resource"),
    )


def packer_validate() -> ResourceType:
    return ResourceType(
        name=Identifier("packer-validator"),
        type=REGISTRY_IMAGE,
        source=RegistryImage(repository="mitodl/concourse-packer-resource"),
    )


def packer_build() -> ResourceType:
    return ResourceType(
        name=Identifier("packer-builder"),
        type=REGISTRY_IMAGE,
        source=RegistryImage(repository="mitodl/concourse-packer-resource-builder"),
    )


def ami_resource() -> ResourceType:
    return ResourceType(
        name=Identifier("amazon-ami"),
        type=REGISTRY_IMAGE,
        source=RegistryImage(repository="jdub/ami-resource"),
    )


def s3_sync() -> ResourceType:
    return ResourceType(
        name=Identifier("s3-sync"),
        type=REGISTRY_IMAGE,
        source=RegistryImage(repository="mitodl/concourse-s3-sync-resource"),
    )


def pulumi_provisioner_resource() -> ResourceType:
    return ResourceType(
        name=Identifier("pulumi-provisioner"),
        type=REGISTRY_IMAGE,
        source=RegistryImage(repository="mitodl/concourse-pulumi-resource-provisioner"),
    )


# https://github.com/arbourd/concourse-slack-alert-resource
# We use only a very basic implementation of this notification framework
def slack_notification_resource() -> ResourceType:
    return ResourceType(
        name=Identifier("slack-notification"),
        type=REGISTRY_IMAGE,
        source=RegistryImage(
            repository="arbourd/concourse-slack-alert-resource", tag="v0.15.0"
        ),
    )



from ol_concourse.lib.constants import REGISTRY_IMAGE
from ol_concourse.lib.models.pipeline import Command, Identifier, TaskConfig, TaskStep


# Generates a TaskStep to perform an instance refresh from a given set
# of filters and queires. The combination of filters and queries should
# be trusted to return one, and only one, autoscale group name.
def instance_refresh_task(
    filters: str,
    queries: str,
) -> TaskStep:
    return TaskStep(
        task=Identifier("instance-refresh"),
        privileged=False,
        config=TaskConfig(
            platform="linux",
            image_resource={
                "type": REGISTRY_IMAGE,
                "source": {"repository": "amazon/aws-cli"},
            },
            params={},
            run=Command(
                path="bash",
                args=[
                    "-ec",
                    f"""ASG_NAME=$(aws autoscaling describe-auto-scaling-groups --color on --no-cli-auto-prompt --no-cli-pager --filters {filters} --query "{queries}" --output text);
                    aws autoscaling start-instance-refresh --color on --no-cli-auto-prompt --no-cli-pager --auto-scaling-group-name $ASG_NAME --preferences MinHealthyPercentage=50,InstanceWarmup=120""",  # noqa: E501
                ],
            ),
        ),
    )


# Generates a TaskStep that can be used to block a job from completing until
# the most recent instance refresh is completed. If no instance refresh is
# found, the task finishes immediately. The combination filters + queries is
# expected to return one and only one autoscale group name.
def block_for_instance_refresh_task(
    filters: str,
    queries: str,
    check_freq: int = 10,
) -> TaskStep:
    return TaskStep(
        task=Identifier("block-for-instance-refresh"),
        privileged=False,
        config=TaskConfig(
            platform="linux",
            image_resource={
                "type": REGISTRY_IMAGE,
                "source": {"repository": "amazon/aws-cli"},
            },
            params={},
            run=Command(
                path="bash",
                args=[
                    "-ec",
                    f""" ASG_NAME=$(aws autoscaling describe-auto-scaling-groups --color on --no-cli-auto-prompt --no-cli-pager --filters {filters} --query "{queries}" --output text);
                    status="InProgress"
                    while [ "$status" = "InProgress" ] || [ "$status" == "Pending" ] || [ "$status" == "Canceling" ]
                    do
                        sleep {check_freq}
                        status=$(aws autoscaling describe-instance-refreshes --color on --no-cli-auto-prompt --no-cli-pager --auto-scaling-group-name $ASG_NAME --query "sort_by(InstanceRefreshes, &StartTime)[].{{Status: Status}}" --output text | tail -n 1)
                        aws autoscaling describe-instance-refreshes --color on --no-cli-auto-prompt --no-cli-pager --auto-scaling-group-name $ASG_NAME --query "sort_by(InstanceRefreshes, &StartTime)[].{{InstanceRefreshId: InstanceRefreshId, StartTime: StartTime, Status: Status}}" --output text | tail -n 1
                    done""",  # noqa: E501
                ],
            ),
        ),
    )
